var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });

// src/search.ts
import axios from "axios";
import { load } from "cheerio";

// src/results.ts
import {
  CurrencyConvertSelector,
  DictionarySearchSelector,
  OrganicSearchSelector,
  TimeSearchSelector,
  TranslateSearchSelector
} from "google-sr-selectors";

// src/constants.ts
var ResultTypes = {
  OrganicResult: "ORGANIC",
  TranslateResult: "TRANSLATE",
  DictionaryResult: "DICTIONARY",
  TimeResult: "TIME",
  CurrencyResult: "CURRENCY"
};

// src/utils.ts
var baseHeaders = {
  Accept: "text/html",
  "Accept-Encoding": "gzip, deflate",
  "Accept-Language": "en-US,en",
  Referer: "https://www.google.com/",
  "upgrade-insecure-requests": 1,
  // the tested user agent is for Chrome 103 on Windows 10
  "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36"
};
function extractUrlFromGoogleLink(googleLink) {
  const match = googleLink.match(/\/url\?q=([^&]+)/);
  if (match) {
    const decodedUrl = decodeURIComponent(match[1]);
    return decodedUrl;
  }
  return null;
}
__name(extractUrlFromGoogleLink, "extractUrlFromGoogleLink");
function prepareRequestConfig(opts) {
  const requestConfig = opts.requestConfig ?? {};
  if (typeof opts.query !== "string")
    throw new TypeError(
      `Search query must be a string, received ${typeof opts.query} instead.`
    );
  if (typeof requestConfig !== "object")
    throw new TypeError(
      `Request config must be an object if specified, received ${typeof requestConfig}.`
    );
  requestConfig.headers = requestConfig.headers ? Object.assign({}, baseHeaders, requestConfig.headers) : baseHeaders;
  requestConfig.url = requestConfig.url ?? "https://www.google.com/search";
  if (!(requestConfig.params instanceof URLSearchParams)) {
    requestConfig.params = new URLSearchParams(requestConfig.params);
  }
  requestConfig.params.set("q", opts.query);
  requestConfig.params.set("gbv", "1");
  requestConfig.responseType = "text";
  return requestConfig;
}
__name(prepareRequestConfig, "prepareRequestConfig");
function throwNoCheerioError(resultParserName) {
  throw new TypeError(
    `CheerioAPI instance is missing, if using as a selector make sure to pass the raw function and not the result of calling it. (ex: [${resultParserName}] instead of [${resultParserName}()])`
  );
}
__name(throwNoCheerioError, "throwNoCheerioError");
function isEmpty(strictSelector, ...values) {
  if (strictSelector)
    return values.some(
      (value) => value === "" || value === void 0 || value === null
    );
  return values.every(
    (value) => value === "" || value === void 0 || value === null
  );
}
__name(isEmpty, "isEmpty");

// src/results.ts
var OrganicResult = /* @__PURE__ */ __name(($, strictSelector) => {
  if (!$)
    throwNoCheerioError("OrganicResult");
  const parsedResults = [];
  const organicSearchBlocks = $(OrganicSearchSelector.block).toArray();
  for (const element of organicSearchBlocks) {
    let link = $(element).find(OrganicSearchSelector.link).attr("href");
    const description = $(element).find(OrganicSearchSelector.description).text();
    const title = $(element).find(OrganicSearchSelector.title).text();
    if (typeof link === "string")
      link = extractUrlFromGoogleLink(link);
    if (isEmpty(strictSelector, link, description, title))
      continue;
    parsedResults.push({
      type: ResultTypes.OrganicResult,
      link,
      description,
      title
    });
  }
  return parsedResults;
}, "OrganicResult");
var TranslateResult = /* @__PURE__ */ __name(($, strictSelector) => {
  if (!$)
    throwNoCheerioError("TranslateResult");
  const sourceLanguage = $(TranslateSearchSelector.sourceLanguage).text().trim();
  const sourceText = $(TranslateSearchSelector.sourceText).val();
  const translationText = $(TranslateSearchSelector.translationText).text().trim();
  const translationLanguage = $(TranslateSearchSelector.targetLanguage).text().trim();
  const translationPronunciation = $(TranslateSearchSelector.pronunciation).text().trim();
  if (isEmpty(
    strictSelector,
    sourceLanguage,
    translationLanguage,
    sourceText,
    translationText,
    translationPronunciation
  ))
    return null;
  return {
    type: ResultTypes.TranslateResult,
    sourceLanguage,
    sourceText,
    translationLanguage,
    translationText,
    translationPronunciation
  };
}, "TranslateResult");
var DictionaryResult = /* @__PURE__ */ __name(($, strictSelector) => {
  if (!$)
    throwNoCheerioError("DictionaryResult");
  const audio = $(DictionarySearchSelector.audio).attr("src") || "";
  const phonetic = $(DictionarySearchSelector.phonetic).first().text().trim();
  const word = $(DictionarySearchSelector.word).text().trim();
  const definitions = [];
  const definitionElements = $(DictionarySearchSelector.definitionPartOfSpeech);
  for (let i = 0; i < definitionElements.length; i++) {
    const partOfSpeech = $(definitionElements[i]).text().trim();
    if (partOfSpeech) {
      const definition = $(DictionarySearchSelector.definition).eq(i).text().trim();
      const example = $(DictionarySearchSelector.definitionExample).eq(i).text().trim();
      const synonyms = $(DictionarySearchSelector.definitionSynonyms).eq(i).text().replace("synonyms: ", "").split(",").map((synonym) => synonym.trim());
      definitions.push({
        partOfSpeech,
        definition,
        example,
        synonyms
      });
    }
  }
  if (isEmpty(strictSelector, audio, phonetic, word))
    return null;
  return {
    type: ResultTypes.DictionaryResult,
    audio,
    phonetic,
    word,
    definitions
  };
}, "DictionaryResult");
var TimeResult = /* @__PURE__ */ __name(($, strictSelector) => {
  if (!$)
    throwNoCheerioError("TimeResult");
  const location = $(TimeSearchSelector.location).text().trim();
  const time = $(TimeSearchSelector.time).text().trim();
  const timeInWords = $(TimeSearchSelector.timeInWords).text().trim();
  if (isEmpty(strictSelector, location, time, timeInWords))
    return null;
  return {
    type: ResultTypes.TimeResult,
    location,
    time,
    timeInWords
  };
}, "TimeResult");
var CurrencyResult = /* @__PURE__ */ __name(($, strictSelector) => {
  if (!$)
    throwNoCheerioError("CurrencyResult");
  const from = $(CurrencyConvertSelector.from).text().replace("=", "").trim();
  const to = $(CurrencyConvertSelector.to).text().trim();
  if (isEmpty(strictSelector, from, to))
    return null;
  return {
    type: ResultTypes.CurrencyResult,
    from,
    to
  };
}, "CurrencyResult");

// src/search.ts
async function search(options) {
  if (!options)
    throw new TypeError(
      `Search options must be provided. Received ${typeof options}`
    );
  const requestConfig = prepareRequestConfig(options);
  const { data } = await axios(requestConfig);
  const cheerioApi = load(data);
  const selectors = options.resultTypes || [OrganicResult];
  let searchResults = [];
  for (const selector of selectors) {
    const result = selector(
      cheerioApi,
      Boolean(options.strictSelector)
    );
    if (result)
      searchResults = searchResults.concat(result);
  }
  return searchResults;
}
__name(search, "search");
async function searchWithPages(options) {
  if (!options)
    throw new TypeError(
      `Search options must be provided. Received ${typeof options}`
    );
  if (typeof options.pages !== "number" && !Array.isArray(options.pages))
    throw new TypeError(
      `Page must be a number or an array of numbers. Received ${typeof options.pages}`
    );
  const searchResults = [];
  const pages = Array.isArray(options.pages) ? options.pages : Array.from({ length: options.pages }, (_, i) => i * 10);
  const baseRequestConfig = prepareRequestConfig(options);
  const selectors = options.resultTypes || [OrganicResult];
  for (const page of pages) {
    baseRequestConfig.params.set("start", String(page));
    const { data } = await axios(baseRequestConfig);
    const cheerioApi = load(data);
    let pageResults = [];
    for (const selector of selectors) {
      const result = selector(
        cheerioApi,
        Boolean(options.strictSelector)
      );
      if (result)
        pageResults = pageResults.concat(result);
    }
    searchResults.push(pageResults);
  }
  return searchResults;
}
__name(searchWithPages, "searchWithPages");
export {
  CurrencyResult,
  DictionaryResult,
  OrganicResult,
  ResultTypes,
  TimeResult,
  TranslateResult,
  search,
  searchWithPages
};
